
# Copyright (c) 1998,2000 University of Southern California.
# All rights reserved.                                            
#                                                                
# Redistribution and use in source and binary forms are permitted
# provided that the above copyright notice and this paragraph are
# duplicated in all such forms and that any documentation, advertising
# materials, and other materials related to such distribution and use
# acknowledge that the software was developed by the University of
# Southern California, Information Sciences Institute.  The name of the
# University may not be used to endorse or promote products derived from
# this software without specific prior written permission.
# 
# THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED
# WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
# 
# $Header: /nfs/jade/vint/CVSROOT/ns-2/tcl/test/test-suite-wireless-lan-newnode-err.tcl,v 1.4 2002/03/08 21:55:44 sfloyd Exp $

proc usage {} {
	global argv0
	puts stderr "usage: ns $argv0 <options> "
	puts "options ...."
	exit 1
}

proc default_options {} {
    global opt
    set opt(chan)	Channel/WirelessChannel
    set opt(prop)       Propagation/Ricean
    #set opt(prop)	Propagation/TwoRayGround
    set opt(netif)	Phy/WirelessPhy
    set opt(err)        UniformErrorProc
    set opt(FECstrength) 0
    set opt(unit)       pkt
    set opt(mac)	Mac/802_11
    set opt(ifq)	Queue/DropTail/PriQueue
    set opt(ll)		LL
    set opt(ant)        Antenna/OmniAntenna
    set opt(x)		670 ;# X dimension of the topography
    set opt(y)		670;# Y dimension of the topography
    set opt(ifqlen)	50	      ;# max packet in ifq
    set opt(seed)	0.0
    set opt(tr)		temp.rands    ;# trace file
    set opt(lm)         "off"          ;# log movement
    #zhibin
    set opt(errrate)    0.25;
    set opt(rp)         dsdv
    set opt(sc)		"../mobility/scene/scen-670x670-50-50.0"
    set opt(nn)		50	      
    set opt(stop)       1000.0
    set opt(cp)         "../mobility/scene/cbr-50-10-5.0.tcl"
}


# =====================================================================
# Other default settings

#LL set mindelay_		50us
#LL set delay_			25us
#LL set bandwidth_		0	;# not used


Queue/DropTail/PriQueue set Prefer_Routing_Protocols    1

# unity gain, omni-directional antennas
# set up the antennas to be centered in the node and 1.5 meters above it
Antenna/OmniAntenna set X_ 0
Antenna/OmniAntenna set Y_ 0
Antenna/OmniAntenna set Z_ 1.5
Antenna/OmniAntenna set Gt_ 1.0
Antenna/OmniAntenna set Gr_ 1.0

# Initialize the SharedMedia interface with parameters to make
# it work like the 914MHz Lucent WaveLAN DSSS radio interface
Phy/WirelessPhy set CPThresh_ 10.0
Phy/WirelessPhy set CSThresh_ 1.559e-11
Phy/WirelessPhy set RXThresh_ 3.652e-10
Phy/WirelessPhy set Rb_ 2*1e6
Phy/WirelessPhy set Pt_ 0.28183815
Phy/WirelessPhy set freq_ 914e+6 
Phy/WirelessPhy set L_ 1.0
Phy/WirelessPhy set bandwidth_ 1e5

# Initialize the Errormodel

#ErrorModule set debug_ false

ErrorModel set enable_ 1
#ErrorModel set markecn_ false
#ErrorModel set bandwidth_ 2Mb
#ErrorModel set rate_ 0.00001
#uniform error rate should get ftom command line

#ErrorModel set rate_ opt(errrate)
#ErrorModel/Trace set good_ 123456789
#ErrorModel/Trace set loss_ 0
#ErrorModel/Periodic set period_ 3.0
#ErrorModel/Periodic set offset_ 0.0
#ErrorModel/Periodic set burstlen_ 0.0
#ErrorModel/MultiState set curperiod_ 0.0
#ErrorModel/MultiState set sttype_ pkt
#ErrorModel/MultiState set texpired_ 0


#================================================
proc getopt {argc argv} {
        global opt
        lappend optlist cp nn seed sc stop tr x y

        for {set i 0} {$i < $argc} {incr i} {
                set arg [lindex $argv $i]
                if {[string range $arg 0 0] != "-"} continue

                set name [string range $arg 1 end]
                set opt($name) [lindex $argv [expr $i+1]]
        }
}

proc UniformErrorProc {} {
    global opt
	
    set	errObj [new ErrorModel]
    $errObj unit packet
    $errObj set rate_ $opt(errrate)
    #puts "rate is $rate_"
    return $errObj
}

proc MultistateErrorProc {} {
    set tmp [new ErrorModel/Uniform 0 pkt]
    set tmp1 [new ErrorModel/Uniform .9 pkt]
    set tmp2 [new ErrorModel/Uniform .5 pkt]
    set m_states [list $tmp $tmp1 $tmp2]
    set m_periods [list 0 .0075 .00375]
    set m_transmx {{0.95 0.05 0} {0 0 1} {1 0 0}}
    set m_trunit pkt
    set m_sttype time
    set m_nstates 3
    set m_nstart [lindex $m_states 0]

    set errObj [new ErrorModel/MultiState $m_states $m_periods $m_transmx $m_trunit $m_sttype $m_nstates $m_nstart]

    $errObj FECstrength 0
    return $errObj
}

proc log-movement {} {
    global logtimer ns_ ns

    set ns $ns_
    source ../mobility/timer.tcl
    Class LogTimer -superclass Timer
    LogTimer instproc timeout {} {
        global opt node_;
        for {set i 0} {$i < $opt(nn)} {incr i} {
            $node_($i) log-movement
        }
        $self sched 0.1
    }

    set logtimer [new LogTimer]
    $logtimer sched 0.1
}

#main ns program
# ======================================================================
# Main Program
# ======================================================================
default_options
getopt $argc $argv

if { $opt(x) == 0 || $opt(y) == 0 } {
        usage $argv0
        exit 1
}

if {$opt(seed) > 0} {
        puts "Seeding Random number generator with $opt(seed)\n"
        ns-random $opt(seed)
}

#
# Initialize Global Variables
#
set ns_         [new Simulator]
set chan        [new $opt(chan)]
set prop        [new $opt(prop)]
set topo        [new Topography]
set tracefd	[open $opt(tr) w]

$ns_ trace-all $tracefd

$topo load_flatgrid $opt(x) $opt(y)

$prop topography $topo

#
# Create God
#
set god_ [create-god $opt(nn)]
#
# log the mobile nodes movements if desired
#
if { $opt(lm) == "on" } {
    log-movement
}
#
#  Create the specified number of nodes $opt(nn) and "attach" them
#  the channel.

   $ns_ node-config -adhocRouting DSDV \
                         -llType $opt(ll) \
                         -macType $opt(mac) \
                         -ifqType $opt(ifq) \
                         -ifqLen $opt(ifqlen) \
                         -antType $opt(ant) \
                         -propType $opt(prop) \
                         -phyType $opt(netif) \
			 -channel [new $opt(chan)] \
			 -topoInstance $topo \
                         -agentTrace ON \
                         -routerTrace OFF \
                         -macTrace ON \
                         -movementTrace OFF 
			# -IncomingErrProc $opt(err)  

if { [string compare $opt(prop) "Propagation/Ricean"  ] == 0 } {
set prop_inst [$ns_ set propInstance_]
$prop_inst MaxVelocity  2.5;
$prop_inst RiceanK        6;
$prop_inst LoadRiceFile  "rice_table.txt";

}
                   
			
#	set node_(0) [$ns_ node]
#        set node_(1) [$ns_ node]
#        set node_(3) [$ns_ node]
#        set node_(19) [$ns_ node]
	
    for {set i 0} {$i < $opt(nn) } {incr i} {
                set node_($i) [$ns_ node]
                $node_($i) random-motion 0              ;# disable random motion
    }
#
# Source the Connection and Movement scripts
#
if { $opt(cp) == "" } {
        puts "*** NOTE: no connection pattern specified."
        set opt(cp) "none"
} else {
        puts "Loading connection pattern..."
        source $opt(cp)
}
#--------------
#
# Tell all the nodes when the simulation ends
#
for {set i 0} {$i < $opt(nn) } {incr i} {
    $ns_ at $opt(stop).000000001 "$node_($i) reset";
}


$ns_ at $opt(stop).00000001 "puts \"NS EXITING...\" ; $ns_ halt"


if { $opt(sc) == "" } {
        puts "*** NOTE: no scenario file specified."
        set opt(sc) "none"
} else {
        puts "Loading scenario file..."
        source $opt(sc)
        puts "Load complete..."
}
#puts $tracefd "M 0.0 nn $opt(nn) x $opt(x) y $opt(y) rp $opt(rp)"
#puts $tracefd "M 0.0 sc $opt(sc) cp $opt(cp) seed $opt(seed)"
#puts $tracefd "M 0.0 prop $opt(prop) ant $opt(ant)"

puts "Starting Simulation..."
$ns_ run








